package org.apidesign.apifest08.currency;

import java.lang.ref.WeakReference;
import java.util.Currency;
import java.util.Map;
import java.util.WeakHashMap;


/**
 * Create convertors using a flyweight to reduce the number of repetative creations of the same convertor.
 * 
 * @author D'Arcy Smith
 * @version 1.0
 */
public final class ConvertorFactory
{
    /**
     * flyweight so that only one vestion of each converter is created at a time.
     */
    private final static Map<String, WeakReference<Convertor>> convertors;
    
    static
    {
        convertors = new WeakHashMap<String, WeakReference<Convertor>>();
    }
    
    /** 
     * Prevent accidental construction.
     */
    private ConvertorFactory()
    {        
    }
    
    /**
     * Get the convertor for the specified currencies.  The currency name format
     * must be acceptable to java.util.Currency.getInstance(String)
     * 
     * @param a the currency to convert from.
     * @param b the currency to convert to.
     * @return the convertor for the specified currencies.
     */
    public static Convertor getConvertor(final String a,
                                         final String b)
    {
        final Currency  currencyA;
        final Currency  currencyB;
        final Convertor convertor;
        
        currencyA = Currency.getInstance(a);
        currencyB = Currency.getInstance(b);        
        convertor = getConvertor(currencyA, currencyB);
        
        return (convertor);
    }
    
    /**
     * Get the convertor for the specified currencies.
     * 
     * @param a the currency to convert from.
     * @param b the currency to convert to.
     * @return the convertor for the specified currencies.
     */
    public static Convertor getConvertor(final Currency a,
                                         final Currency b)
    {
        final String key;        
        Convertor    convertor;

        if(a == null)
        {
            throw new IllegalArgumentException("a cannot be null");
        }

        if(b == null)
        {
            throw new IllegalArgumentException("b cannot be null");
        }

        key = a.getCurrencyCode() + b.getCurrencyCode();

        // make sure that we don't try to overwrite one
        synchronized(convertors)
        {
            if(!(convertors.containsKey(key)))
            {        
                convertor = new Convertor(a, b);
                convertors.put(key, new WeakReference(convertor));
            }
        }

        convertor = convertors.get(key).get();
        
        return (convertor);
    }
}
